'Args:
'(0):  LDAP://path...
'(1):  LDAP Filter
'(2):  LDAP properties delimited with a comma
'(3):  onelevel or subtree
'(4):  Temporary file path

' Setup ADO objects.
Set adoCommand = CreateObject("ADODB.Command")
Set adoConnection = CreateObject("ADODB.Connection")
adoConnection.Provider = "ADsDSOObject"
adoConnection.Open "Active Directory Provider"
Set adoCommand.ActiveConnection = adoConnection

strBase = "<" & Wscript.Arguments(0) & ">"

' Filter on user objects.
strFilter = "(" & WScript.Arguments(1) & ")"

' Comma delimited list of attribute values to retrieve.
strAttributes = WScript.Arguments(2)

arrAttributes = Split(WScript.Arguments(2), ",")


If Instr(UCase(strAtributes), "DISTINGUISHEDNAME") = 0 Then
	strAttributes = strAttributes & ",distinguishedName"
End If

If Instr(UCase(strAtributes), "CN") = 0 Then
	strAttributes = strAttributes & ",CN"
End If


If Not Instr(UCase(strAtributes), "CANONICALNAME") = 0 Then
	strAttributes = Left(strAttributes, InStr(UCase(strAtributes), "CANONICALNAME", 1)) &  _ 
	Right(strAttributes, InStr(UCase(strAtributes), "CANONICALNAME", 1) + Len("CANONICALNAME"))
End If

Replace strAttributes, ",,", ","

' Construct the LDAP syntax query.
strQuery = strBase & ";" & strFilter & ";" & strAttributes & ";" & WScript.Arguments(3)

adoCommand.CommandText = strQuery
'TODO:  try
' oCommand.CommandText = "SELECT samAccountName FROM '" & searchpath & "' WHERE objectClass='user' AND samAccountName = '" & strSearchUser & "'"

adoCommand.Properties("Page Size") = 100
adoCommand.Properties("Timeout") = 120
adoCommand.Properties("Cache Results") = False

' Run the query.
Err.Number = 0
On Error Resume Next
Set adoRecordset = adoCommand.Execute

If Not Err.Number = 0 Then
	MsgBox "Error searching AD.  This may be the result of manually adding invalid LDAP properties to the search fields" _
	& vbCrLf  & vbCrLf  & strQuery
	
	WScript.Quit
End If

Set objFSO = CreateObject("Scripting.FileSystemObject")

Set objTxt = objFSO.CreateTextFile(WScript.Arguments(4))

' Enumerate the resulting recordset.
Do Until adoRecordset.EOF
    ' Retrieve values and display.
    strResult = ""
    
    GetDomain strResult, adoRecordset.Fields("distinguishedName").Value
    
    strResult = strResult & vbTab
    
    strObjectClass = ""
	GetArrayLast strObjectClass, adoRecordset.Fields("objectClass").Value
	
	strObjectClassU = UCase(strObjectClass)
	
    
    For Each strAttr In arrAttributes
    	On Error Resume Next
    	strAttrU = UCase(strAttr)
    	  
    	'Ensure that we get the correct identity reference for NT Account name
    	If strAttrU = "SAMACCOUNTNAME" Then    		
    		Select Case strObjectClassU
    			Case "COMPUTER", "CONTAINER", "CONTACT"
    				'GetArrayLast strResult,  adoRecordset.Fields("CN")
    				strResult = strResult & adoRecordset.Fields("CN")    
    			Case "CONTAINER"
    				GetCNfromDN strResult, adoRecordset.Fields("distinguishedName")
    			Case "ORGANIZATIONALUNIT"    		    				
    				GetOUfromDN strResult, adoRecordset.Fields("distinguishedName")
    			Case "USER", "GROUP"
    				strResult = strResult &  adoRecordset.Fields("samAccountName")   			
    			Case Else    				
    				strResult = strResult & adoRecordset.Fields("CN")
    		End Select	
    	'Special case attributes that aren't easily converted to strings
    	ElseIf strAttrU = "DESCRIPTION" Then
    		arrDesc = adoRecordset.Fields(strAttr).Value
    		strResult = strResult & arrDesc(0)
    	ElseIf strAttrU = "OBJECTCLASS" Then    		
    		strResult = strResult & strObjectClass
    	ElseIf strAttrU = "GROUPTYPE" Then
    		ConvertGroupType strResult, adoRecordset.Fields(strAttr).Value
    	ElseIf (strAttrU = "LASTLOGONTIMESTAMP") Or (strAttrU = "LASTLOGON") Or (strAttrU = "PWDLASTSET") Then
    		ConvertFileTime strResult, adoRecordset.Fields(strAttr).Value    	
    	ElseIf (strAttrU = "CANONICALNAME") Then
    		 convertDNtoCanonical strResult, adoRecordset.Fields("distinguishedName").Value    	
    	'ElseIf ...other special cases
    	Else    		
    		'strResult = strResult & adoRecordset.Fields(strAttr)
    		RemoveNewLinesAndTabs strResult, adoRecordset.Fields(strAttr)
    	End If
    	
    	strResult = strResult & vbTab    	
    Next
    
    objTxt.WriteLine(strResult)
    'Wscript.Echo strResult
    ' Move to the next record in the recordset.
    adoRecordset.MoveNext
Loop

' Clean up.
objTxt.Close
Set objFSO = Nothing
adoRecordset.Close
adoConnection.Close 
WScript.Quit(0)

Sub convertDNtoCanonical (strResult, DN)
	
	strArr1 = Split(DN, ",DC=")
	
	For i = 1 To UBound(strArr1)
		If Not i = 1 Then
			strResult = strResult & "." & strArr1(i)
		Else
			strResult = strResult & strArr1(i)
		End If
	Next
	
	'Trim the OU=/CN=
	strOUpath = Right(strArr1(0), Len(strArr1(0)) - 3)
	
	strArr2 = Split(strOUpath, ",CN=")
	
	strPath = ""
	
	If UBound(strArr2) > 0 Then
		For i2 = 0 To UBound(strArr2)
			If i2 = 0 Then
				strPath = strArr2(i2)
			Else
				strPath = strArr2(i2) & "/" & strPath
			End If			
		Next
	Else
		strArr3 = Split(strOUpath, ",OU=")
		For i3 = 0 To UBound(strArr3)
			If i3 = 0 Then
				strPath = strArr3(i3)
			Else
				strPath = strArr3(i3) & "/" & strPath
			End If
		Next
	End If

	strResult = strResult & "/" & strPath
		
End Sub

Sub GetDomain(strResult, DN)
	strArr = Split(DN, ",DC=")
	
	For i = 1 To UBound(strArr)
		If Not i = 1 Then
			strResult = strResult & "." & strArr(i)
		Else
			strResult = strArr(i)
		End If
	Next
	
End Sub

Sub GetOUfromDN (strResult, strDN)	
	arr1 = Split(strDN, "OU=")
	arr2 = Split(arr1(1), ",")	
	strResult = strResult & arr2(0) & " | path: " & strDN
End Sub

Sub GetCNfromDN (strResult, strDN)	
	arr1 = Split(strDN, "CN=")
	arr2 = Split(arr1(1), ",")	
	strResult = strResult & arr2(0) & " | path: " & strDN
End Sub

Sub GetArrayLast (strResult, arrInput)
	strResult = strResult & arrInput(UBound(arrInput))
End Sub

Sub RemoveNewLinesAndTabs (strResult, strInput)
	strInput = Replace(strInput, vbCrLf, "<newLine>")
	strInput = Replace(strInput, vbTab, "<TAB>")
	strResult = strResult & strInput
End Sub

Sub ConvertGroupType (strResult, groupType)
	
	Select Case groupType
		Case 8
			strResult = strResult & "Distribution-Universal"
		Case -2147483640
			strResult = strResult &"Security-Universal"
		Case 2
			strResult = strResult & "Distribution-Global"
		Case -2147483646
			strResult = strResult & "Security-Global"
		Case 4
			strResult = strResult & "Distribution-DomainLocal"
		Case -2147483644
			strResult = strResult & "Security-DomainLocal"	
	End Select

End Sub

Sub ConvertFileTime (strResult, objDate)
	strDate = "Never"
	dtmDate = #1/1/1601#
    lngHigh = objDate.HighPart
    lngLow = objDate.LowPart
    If (lngLow < 0) Then
        lngHigh = lngHigh + 1
    End If
    If (lngHigh = 0) And (lngLow = 0) Then
        dtmDate = #1/1/1601#
    Else
        dtmDate = #1/1/1601# + (((lngHigh * (2 ^ 32)) _
            + lngLow)/600000000 - lngBias)/1440
        strDate = dtmDate
    End If
    
    strResult = strResult & strDate 'dtmDate

End Sub